Class: BGL::Vertex

Inherits:
Object show all
Included in:
Roby::DirectedRelationSupport
Defined in:
ext/graph/algorithm.cc

Overview

A module to be mixed in objects used as vertices in Graph. It allows to use the same object in more than one graph.