Class: DRbObject
Direct Known Subclasses
Instance Method Summary collapse
-
#remote_id ⇒ Object
Converts this DRbObject into Roby::Distributed::RemoteID.
-
#to_s ⇒ Object
We don’t want this method to call the remote object.
Instance Method Details
#remote_id ⇒ Object
Converts this DRbObject into Roby::Distributed::RemoteID
61 62 63 |
# File 'lib/roby/distributed/base.rb', line 61 def remote_id @__droby_remote_id__ ||= Roby::Distributed::RemoteID.new(__drburi, __drbref) end |
#to_s ⇒ Object
We don’t want this method to call the remote object.
57 58 59 |
# File 'lib/roby/distributed/base.rb', line 57 def to_s inspect end |