Exception: Roby::Actions::Models::InterfaceBase::ArgumentCountMismatch
- Inherits:
-
ScriptError
- Object
- ScriptError
- Roby::Actions::Models::InterfaceBase::ArgumentCountMismatch
- Defined in:
- lib/roby/actions/models/interface_base.rb
Overview
Exception raised when an action is defined with the wrong count of argument (zero if no arguments are specified and one if arguments are specified)