Method: Roby::BasicObject#sibling_of

Defined in:
lib/roby/basic_object.rb

#sibling_of(remote_object, peer) ⇒ Object

Sets remote_object as the remote siblings for self on peer, and notifies peer that self is the remote siblings for remote_object



53
54
55
56
57
58
59
60
# File 'lib/roby/basic_object.rb', line 53

def sibling_of(remote_object, peer)
    if !distribute?
	raise ArgumentError, "#{self} is local only"
    end

    add_sibling_for(peer, remote_object)
    peer.transmit(:added_sibling, remote_object, remote_id)
end