Module: Roby::Distributed::EventNotifications::PlanCacheCleanup

Included in:
Plan
Defined in:
lib/roby/distributed/notifications.rb

Overview

This module define hooks on Roby::Plan to manage the event fired cache. It is required by the receiving side of the event propagation distribution.

See PeerServer#pending_events

Instance Method Summary collapse

Instance Method Details

#finalized_event(generator) ⇒ Object

Removes events generated by generator from the Event object cache, PeerServer#pending_events. This cache is used by PeerServer#event_for on behalf of PeerServer#event_fired and PeerServer#event_add_propagation



386
387
388
389
390
391
# File 'lib/roby/distributed/notifications.rb', line 386

def finalized_event(generator)
    super if defined? super
    Distributed.peers.each_value do |peer|
        peer.local_server.pending_events.delete(generator)
    end
end