Exception: Roby::Distributed::NotEditor

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/roby/distributed/transaction.rb

Overview

Raised when an operation needs the edition token, while the local plan manager does not have it.