Exception: Roby::Distributed::NotReady

Inherits:
RuntimeError
  • Object
show all
Defined in:
lib/roby/distributed/transaction.rb

Overview

Raised when a commit is attempted while the transaction is not ready, i.e. the token should be passed once more in the edition ring.