Class: Roby::EventConstraints::UnboundTaskPredicate::CompiledPredicate
- Defined in:
- lib/roby/event_constraints.rb
Overview
See #compile.
Objects of this class hold the compiled predicate used for evaluation
See #compile.
Objects of this class hold the compiled predicate used for evaluation