Class: Roby::Models::Arguments::Argument
- Defined in:
- lib/roby/models/arguments.rb
Overview
Representation of one argument
Instance Attribute Summary collapse
-
#default ⇒ Object
Returns the value of attribute default.
-
#doc ⇒ Object
Returns the value of attribute doc.
-
#name ⇒ Object
Returns the value of attribute name.
Instance Method Summary collapse
-
#has_default? ⇒ Boolean
Tests whether this argument has a default.
-
#has_delayed_default? ⇒ Boolean
Tests whether this argument has a delayed argument as default.
Instance Attribute Details
#default ⇒ Object
Returns the value of attribute default
9 10 11 |
# File 'lib/roby/models/arguments.rb', line 9 def default @default end |
#doc ⇒ Object
Returns the value of attribute doc
9 10 11 |
# File 'lib/roby/models/arguments.rb', line 9 def doc @doc end |
#name ⇒ Object
Returns the value of attribute name
9 10 11 |
# File 'lib/roby/models/arguments.rb', line 9 def name @name end |
Instance Method Details
#has_default? ⇒ Boolean
Tests whether this argument has a default
11 12 13 |
# File 'lib/roby/models/arguments.rb', line 11 def has_default? default != NO_DEFAULT_ARGUMENT end |
#has_delayed_default? ⇒ Boolean
Tests whether this argument has a delayed argument as default
16 17 18 |
# File 'lib/roby/models/arguments.rb', line 16 def has_delayed_default? has_default? && TaskArguments.delayed_argument?(default) end |