Method: Roby::Planning::MethodModel#returns

Defined in:
lib/roby/planning/model.rb

#returnsObject

The return type the method model defines

If this is nil, methods of this model may return a task array or a task aggregation



145
# File 'lib/roby/planning/model.rb', line 145

def returns;    options[:returns] end