Module: Roby::Propagation::ExecutablePlanChanged

Defined in:
lib/roby/propagation.rb

Overview

This module hooks in plan modifications to clear the event ordering cache (Propagation.event_ordering) when needed.

It is included in the main plan by Control#initialize

Instance Method Summary collapse

Instance Method Details

#discovered_events(objects) ⇒ Object



238
239
240
241
# File 'lib/roby/propagation.rb', line 238

def discovered_events(objects)
    super if defined? super
    Roby::Propagation.event_ordering.clear
end

#discovered_tasks(objects) ⇒ Object



242
243
244
245
# File 'lib/roby/propagation.rb', line 242

def discovered_tasks(objects)
    super if defined? super
    Roby::Propagation.event_ordering.clear
end