Class: Roby::TaskStructure::PlannedBy
- Defined in:
- lib/roby/task_structure/planned_by.rb
Defined Under Namespace
Modules: Extension
Instance Method Summary collapse
-
#check_structure(plan) ⇒ Object
Returns a set of PlanningFailedError exceptions for all abstract tasks for which planning has failed.
Instance Method Details
#check_structure(plan) ⇒ Object
Returns a set of PlanningFailedError exceptions for all abstract tasks for which planning has failed
42 43 44 45 46 47 48 49 50 51 52 53 54 55 |
# File 'lib/roby/task_structure/planned_by.rb', line 42 def check_structure(plan) result = [] each_edge do |planned_task, planning_task, | next if plan != planning_task.plan next if !planning_task.failed? next if !planned_task.self_owned? if (planned_task.pending? && !planned_task.executable?) || ![:optional] result << [Roby::PlanningFailedError.new(planned_task, planning_task), nil] end end result end |