Class: Roby::Test::ExecutionExpectations::Expectation
- Defined in:
- lib/roby/test/execution_expectations.rb
Overview
Null implementation of an expectation
Direct Known Subclasses
Achieve, BecomeUnreachable, EmitGenerator, EmitGeneratorModel, ErrorExpectation, FailsToStart, Finalize, HaveFrameworkError, Maintain, NotBecomeUnreachable, NotEmitGenerator, NotEmitGeneratorModel, NotFinalize, PromiseFinishes, Quarantine
Instance Attribute Summary collapse
-
#backtrace ⇒ Object
readonly
Returns the value of attribute backtrace.
Instance Method Summary collapse
- #explain_unachievable(propagation_info) ⇒ Object
-
#initialize(backtrace) ⇒ Expectation
constructor
A new instance of Expectation.
- #relates_to_error?(error) ⇒ Boolean
- #unachievable?(propagation_info) ⇒ Boolean
-
#update_match(propagation_info) ⇒ Object
Verifies whether the expectation is met at this point.
Constructor Details
#initialize(backtrace) ⇒ Expectation
Returns a new instance of Expectation.
675 676 677 |
# File 'lib/roby/test/execution_expectations.rb', line 675 def initialize(backtrace) @backtrace = backtrace end |
Instance Attribute Details
#backtrace ⇒ Object (readonly)
Returns the value of attribute backtrace.
673 674 675 |
# File 'lib/roby/test/execution_expectations.rb', line 673 def backtrace @backtrace end |
Instance Method Details
#explain_unachievable(propagation_info) ⇒ Object
688 689 690 |
# File 'lib/roby/test/execution_expectations.rb', line 688 def explain_unachievable(propagation_info) nil end |
#relates_to_error?(error) ⇒ Boolean
691 692 693 |
# File 'lib/roby/test/execution_expectations.rb', line 691 def relates_to_error?(error) false end |
#unachievable?(propagation_info) ⇒ Boolean
685 686 687 |
# File 'lib/roby/test/execution_expectations.rb', line 685 def unachievable?(propagation_info) false end |
#update_match(propagation_info) ⇒ Object
Verifies whether the expectation is met at this point
This method is meant to update
682 683 684 |
# File 'lib/roby/test/execution_expectations.rb', line 682 def update_match(propagation_info) true end |