Class: Roby::TimeDeltaEvent
- Inherits:
-
DeltaEvent
- Object
- BasicObject
- PlanObject
- EventGenerator
- StateEvent
- DeltaEvent
- Roby::TimeDeltaEvent
- Defined in:
- lib/roby/state/events.rb
Overview
An event which emits at a given period (delta in time)
Constant Summary
Constants included from Log::EventGeneratorHooks
Log::EventGeneratorHooks::HOOKS
Constants included from Log::BasicObjectHooks
Instance Attribute Summary
Attributes inherited from DeltaEvent
Attributes inherited from EventGenerator
#command, #executable, #pending, #unreachable_handlers
Attributes inherited from PlanObject
#executable, #plan, #removed_at
Attributes inherited from BasicObject
Instance Method Summary collapse
-
#delta ⇒ Object
Returns how much time elapsed since the last emission.
-
#has_sample? ⇒ Boolean
Always true, as we can always measure time.
-
#read ⇒ Object
Returns the current time.
Methods inherited from DeltaEvent
event_types, #or, or, #poll, register_as, #reset
Methods inherited from StateEvent
#disable, #disabled?, #enable, #enabled?
Methods inherited from EventGenerator
#&, #_dump, _load, #achieve_with, #add_child_object, #call, #call_handlers, #call_without_propagation, #called, #calling, #cancel, #controlable?, #default_command, #delay, #droby_dump, #each_precondition, #emit, #emit_failed, #emit_on, #emit_without_propagation, #emitting, event_gathering, #executable?, #filter, #fired, #forward, #forward_once, #forwarding, gather_events, #if_unreachable, #initialize, #initialize_copy, #last, #model, #name, #new, #on, #once, #pending?, #postpone, #postponed, #precondition, #pretty_print, #realize_with, #related_events, #related_tasks, remove_event_gathering, #signal, #signal_once, #signalling, #to_event, #unreachable!, #until, #when_unreachable, #|
Methods included from Distributed::DRobyModel::Dump
Methods included from Distributed::EventNotifications
#fired, #forwarding, #signalling
Methods included from Propagation::EventPrecedenceChanged
#added_child_object, #removed_child_object
Methods included from Log::EventGeneratorHooks
#added_child_object, #called, #calling, #emitting, #fired, #forwarding, #postponed, #removed_child_object, #signalling
Methods inherited from PlanObject
#add_child_object, #apply_relation_changes, child_plan_object, #each_plan_child, #executable?, #finalized?, #forget_peer, #read_write?, #remotely_useful?, #removing_child_object, #replace_by, #replace_subplan_by, #root_object, #root_object?, #subscribed?, #update_on?, #updated_by?
Methods included from Distributed::RelationModificationHooks
#added_child_object, #removed_child_object
Methods included from Roby::Transactions::PlanObjectUpdates
#adding_child_object, #removing_child_object
Methods included from DirectedRelationSupport
#add_child_object, #add_parent_object, #check_is_relation, #related_objects, #relations, #remove_child_object, #remove_children, #remove_parent_object, #remove_parents, #remove_relations
Methods inherited from BasicObject
#add_sibling_for, #distribute?, distribute?, #finalized?, #forget_peer, #has_sibling_on?, #initialize_copy, local_only, #read_write?, #remotely_useful?, #remove_sibling_for, #self_owned?, #sibling_of, #sibling_on, #subscribe, #subscribed?, #update_on?, #updated?, #updated_by?, #updated_peers
Methods included from Log::BasicObjectHooks
Constructor Details
This class inherits a constructor from Roby::EventGenerator
Instance Method Details
#delta ⇒ Object
Returns how much time elapsed since the last emission
189 |
# File 'lib/roby/state/events.rb', line 189 def delta; Time.now - last_value end |
#has_sample? ⇒ Boolean
Always true, as we can always measure time
187 |
# File 'lib/roby/state/events.rb', line 187 def has_sample?; true end |
#read ⇒ Object
Returns the current time
191 |
# File 'lib/roby/state/events.rb', line 191 def read; Time.now end |