Module: Roby::Transaction::Proxying::Cache

Included in:
Models::PlanObject, PlanService
Defined in:
lib/roby/transaction/proxying.rb

Instance Attribute Summary collapse

Instance Attribute Details

#transaction_forwarder_moduleModule?

The module used to transform proxies into call forwarder at commit time

It is initialized and used by Transaction::Proxying#forwarder_module_for



26
27
28
# File 'lib/roby/transaction/proxying.rb', line 26

def transaction_forwarder_module
  @transaction_forwarder_module
end

#transaction_proxy_moduleModule?

The full module used to build transaction proxies for objects of this model

It is initialized and used by Transaction::Proxying#proxying_module_for



35
36
37
# File 'lib/roby/transaction/proxying.rb', line 35

def transaction_proxy_module
  @transaction_proxy_module
end