Class: Ui_ReplayControls

Inherits:
Object show all
Defined in:
lib/roby/log/gui/replay_controls_ui.rb

Overview

** Form generated from reading ui file ‘replay_controls.ui’ ** ** Created: Sat Jun 14 16:46:54 2008 ** by: Qt User Interface Compiler version 4.4.0 ** ** WARNING! All changes made in this file will be lost when recompiling ui file!

Direct Known Subclasses

Ui::ReplayControls

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#actionBookmarksAddObject (readonly)

Returns the value of attribute actionBookmarksAdd.



11
12
13
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 11

def actionBookmarksAdd
  @actionBookmarksAdd
end

#actionBookmarksLoadObject (readonly)

Returns the value of attribute actionBookmarksLoad.



15
16
17
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 15

def actionBookmarksLoad
  @actionBookmarksLoad
end

#actionBookmarksSaveObject (readonly)

Returns the value of attribute actionBookmarksSave.



13
14
15
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 13

def actionBookmarksSave
  @actionBookmarksSave
end

#actionBookmarksSaveAsObject (readonly)

Returns the value of attribute actionBookmarksSaveAs.



14
15
16
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 14

def actionBookmarksSaveAs
  @actionBookmarksSaveAs
end

#actionBookmarksSetMarkObject (readonly)

Returns the value of attribute actionBookmarksSetMark.



16
17
18
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 16

def actionBookmarksSetMark
  @actionBookmarksSetMark
end

#actionGotoObject (readonly)

Returns the value of attribute actionGoto.



12
13
14
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 12

def actionGoto
  @actionGoto
end

#bookmarksObject (readonly)

Returns the value of attribute bookmarks.



31
32
33
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 31

def bookmarks
  @bookmarks
end

#fasterObject (readonly)

Returns the value of attribute faster.



29
30
31
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 29

def faster
  @faster
end

#gotoObject (readonly)

Returns the value of attribute goto.



22
23
24
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 22

def goto
  @goto
end

#grp_playObject (readonly)

Returns the value of attribute grp_play.



18
19
20
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 18

def grp_play
  @grp_play
end

#hboxLayoutObject (readonly)

Returns the value of attribute hboxLayout.



20
21
22
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 20

def hboxLayout
  @hboxLayout
end

#hboxLayout1Object (readonly)

Returns the value of attribute hboxLayout1.



24
25
26
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 24

def hboxLayout1
  @hboxLayout1
end

#hboxLayout2Object (readonly)

Returns the value of attribute hboxLayout2.



30
31
32
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 30

def hboxLayout2
  @hboxLayout2
end

#playObject (readonly)

Returns the value of attribute play.



28
29
30
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 28

def play
  @play
end

#play_stepObject (readonly)

Returns the value of attribute play_step.



34
35
36
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 34

def play_step
  @play_step
end

#progressObject (readonly)

Returns the value of attribute progress.



21
22
23
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 21

def progress
  @progress
end

#seek_startObject (readonly)

Returns the value of attribute seek_start.



33
34
35
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 33

def seek_start
  @seek_start
end

#slowerObject (readonly)

Returns the value of attribute slower.



27
28
29
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 27

def slower
  @slower
end

#spacerItemObject (readonly)

Returns the value of attribute spacerItem.



32
33
34
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 32

def spacerItem
  @spacerItem
end

#speedObject (readonly)

Returns the value of attribute speed.



26
27
28
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 26

def speed
  @speed
end

#textLabel1Object (readonly)

Returns the value of attribute textLabel1.



25
26
27
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 25

def textLabel1
  @textLabel1
end

#time_lcdObject (readonly)

Returns the value of attribute time_lcd.



23
24
25
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 23

def time_lcd
  @time_lcd
end

#vboxLayoutObject (readonly)

Returns the value of attribute vboxLayout.



17
18
19
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 17

def vboxLayout
  @vboxLayout
end

#vboxLayout1Object (readonly)

Returns the value of attribute vboxLayout1.



19
20
21
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 19

def vboxLayout1
  @vboxLayout1
end

Instance Method Details

#retranslate_ui(replayControls) ⇒ Object

retranslateUi



219
220
221
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 219

def retranslate_ui(replayControls)
    retranslateUi(replayControls)
end

#retranslateUi(replayControls) ⇒ Object



171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 171

def retranslateUi(replayControls)
replayControls.windowTitle = Qt::Application.translate("ReplayControls", "Form", nil, Qt::Application::UnicodeUTF8)
@actionBookmarksAdd.text = Qt::Application.translate("ReplayControls", "Add...", nil, Qt::Application::UnicodeUTF8)
@actionGoto.text = Qt::Application.translate("ReplayControls", "goto", nil, Qt::Application::UnicodeUTF8)
@actionGoto.shortcut = Qt::Application.translate("ReplayControls", "Shift+G, G", nil, Qt::Application::UnicodeUTF8)
@actionBookmarksSave.text = Qt::Application.translate("ReplayControls", "Save", nil, Qt::Application::UnicodeUTF8)
@actionBookmarksSaveAs.text = Qt::Application.translate("ReplayControls", "Save As", nil, Qt::Application::UnicodeUTF8)
@actionBookmarksLoad.text = Qt::Application.translate("ReplayControls", "Load", nil, Qt::Application::UnicodeUTF8)
@actionBookmarksSetMark.text = Qt::Application.translate("ReplayControls", "Set mark", nil, Qt::Application::UnicodeUTF8)
@grp_play.title = Qt::Application.translate("ReplayControls", "Play", nil, Qt::Application::UnicodeUTF8)
@goto.text = Qt::Application.translate("ReplayControls", "&Goto", nil, Qt::Application::UnicodeUTF8)
@textLabel1.text = Qt::Application.translate("ReplayControls", "Speed", nil, Qt::Application::UnicodeUTF8)
@speed.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-weight:600;\">Current playing speed</span></p>\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-weight:600;\"><span style=\" font-weight:400;\">it is the factor between the time in the log file and the time of replay. For instance, 2 means that we replay twice as fast as in reality.</span></p>\n" \
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p></body></html>", nil, Qt::Application::UnicodeUTF8)
@speed.text = Qt::Application.translate("ReplayControls", "1", nil, Qt::Application::UnicodeUTF8)
@slower.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Decrease speed</p></body></html>", nil, Qt::Application::UnicodeUTF8)
@slower.text = Qt::Application.translate("ReplayControls", "<<", nil, Qt::Application::UnicodeUTF8)
@play.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Play</p></body></html>", nil, Qt::Application::UnicodeUTF8)
@play.text = Qt::Application.translate("ReplayControls", ">", nil, Qt::Application::UnicodeUTF8)
@faster.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Increase speed</p></body></html>", nil, Qt::Application::UnicodeUTF8)
@faster.text = Qt::Application.translate("ReplayControls", ">>", nil, Qt::Application::UnicodeUTF8)
@bookmarks.text = Qt::Application.translate("ReplayControls", "Bookmarks", nil, Qt::Application::UnicodeUTF8)
@seek_start.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Go to start</p></body></html>", nil, Qt::Application::UnicodeUTF8)
@seek_start.text = Qt::Application.translate("ReplayControls", "|<<", nil, Qt::Application::UnicodeUTF8)
@seek_start.shortcut = ''
@play_step.toolTip = Qt::Application.translate("ReplayControls", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n" \
"p, li { white-space: pre-wrap; }\n" \
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal; text-decoration:none;\">\n" \
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Go forward one step</p></body></html>", nil, Qt::Application::UnicodeUTF8)
@play_step.text = Qt::Application.translate("ReplayControls", ">|", nil, Qt::Application::UnicodeUTF8)
end

#setup_ui(replayControls) ⇒ Object

setupUi



167
168
169
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 167

def setup_ui(replayControls)
    setupUi(replayControls)
end

#setupUi(replayControls) ⇒ Object



36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
# File 'lib/roby/log/gui/replay_controls_ui.rb', line 36

def setupUi(replayControls)
if replayControls.objectName.nil?
    replayControls.objectName = "replayControls"
end
replayControls.resize(432, 168)
@actionBookmarksAdd = Qt::Action.new(replayControls)
@actionBookmarksAdd.objectName = "actionBookmarksAdd"
@actionGoto = Qt::Action.new(replayControls)
@actionGoto.objectName = "actionGoto"
@actionGoto.shortcutContext = Qt::ApplicationShortcut
@actionBookmarksSave = Qt::Action.new(replayControls)
@actionBookmarksSave.objectName = "actionBookmarksSave"
@actionBookmarksSave.enabled = false
@actionBookmarksSaveAs = Qt::Action.new(replayControls)
@actionBookmarksSaveAs.objectName = "actionBookmarksSaveAs"
@actionBookmarksSaveAs.enabled = true
@actionBookmarksLoad = Qt::Action.new(replayControls)
@actionBookmarksLoad.objectName = "actionBookmarksLoad"
@actionBookmarksSetMark = Qt::Action.new(replayControls)
@actionBookmarksSetMark.objectName = "actionBookmarksSetMark"
@vboxLayout = Qt::VBoxLayout.new(replayControls)
@vboxLayout.objectName = "vboxLayout"
@grp_play = Qt::GroupBox.new(replayControls)
@grp_play.objectName = "grp_play"
@vboxLayout1 = Qt::VBoxLayout.new(@grp_play)
@vboxLayout1.objectName = "vboxLayout1"
@hboxLayout = Qt::HBoxLayout.new()
@hboxLayout.objectName = "hboxLayout"
@progress = Qt::ProgressBar.new(@grp_play)
@progress.objectName = "progress"
@progress.value = 0
@progress.orientation = Qt::Horizontal

@hboxLayout.addWidget(@progress)

@goto = Qt::PushButton.new(@grp_play)
@goto.objectName = "goto"

@hboxLayout.addWidget(@goto)

@time_lcd = Qt::LCDNumber.new(@grp_play)
@time_lcd.objectName = "time_lcd"
@time_lcd.numDigits = 6
@time_lcd.segmentStyle = Qt::LCDNumber::Flat
@time_lcd.setProperty("value", Qt::Variant.new(0))

@hboxLayout.addWidget(@time_lcd)


@vboxLayout1.addLayout(@hboxLayout)

@hboxLayout1 = Qt::HBoxLayout.new()
@hboxLayout1.objectName = "hboxLayout1"
@textLabel1 = Qt::Label.new(@grp_play)
@textLabel1.objectName = "textLabel1"
@textLabel1.wordWrap = false

@hboxLayout1.addWidget(@textLabel1)

@speed = Qt::LineEdit.new(@grp_play)
@speed.objectName = "speed"
@sizePolicy = Qt::SizePolicy.new(Qt::SizePolicy::Minimum, Qt::SizePolicy::Fixed)
@sizePolicy.setHorizontalStretch(1)
@sizePolicy.setVerticalStretch(0)
@sizePolicy.heightForWidth = @speed.sizePolicy.hasHeightForWidth
@speed.sizePolicy = @sizePolicy
@speed.maximumSize = Qt::Size.new(80, 32767)
@speed.alignment = Qt::AlignRight

@hboxLayout1.addWidget(@speed)

@slower = Qt::PushButton.new(@grp_play)
@slower.objectName = "slower"

@hboxLayout1.addWidget(@slower)

@play = Qt::PushButton.new(@grp_play)
@play.objectName = "play"
@sizePolicy1 = Qt::SizePolicy.new(Qt::SizePolicy::MinimumExpanding, Qt::SizePolicy::Fixed)
@sizePolicy1.setHorizontalStretch(2)
@sizePolicy1.setVerticalStretch(0)
@sizePolicy1.heightForWidth = @play.sizePolicy.hasHeightForWidth
@play.sizePolicy = @sizePolicy1
@play.checkable = true

@hboxLayout1.addWidget(@play)

@faster = Qt::PushButton.new(@grp_play)
@faster.objectName = "faster"

@hboxLayout1.addWidget(@faster)


@vboxLayout1.addLayout(@hboxLayout1)

@hboxLayout2 = Qt::HBoxLayout.new()
@hboxLayout2.objectName = "hboxLayout2"
@bookmarks = Qt::ToolButton.new(@grp_play)
@bookmarks.objectName = "bookmarks"
@bookmarks.popupMode = Qt::ToolButton::InstantPopup
@bookmarks.toolButtonStyle = Qt::ToolButtonTextBesideIcon
@bookmarks.arrowType = Qt::DownArrow

@hboxLayout2.addWidget(@bookmarks)

@spacerItem = Qt::SpacerItem.new(40, 20, Qt::SizePolicy::Expanding, Qt::SizePolicy::Minimum)

@hboxLayout2.addItem(@spacerItem)

@seek_start = Qt::PushButton.new(@grp_play)
@seek_start.objectName = "seek_start"

@hboxLayout2.addWidget(@seek_start)

@play_step = Qt::PushButton.new(@grp_play)
@play_step.objectName = "play_step"

@hboxLayout2.addWidget(@play_step)


@vboxLayout1.addLayout(@hboxLayout2)


@vboxLayout.addWidget(@grp_play)


retranslateUi(replayControls)

Qt::MetaObject.connectSlotsByName(replayControls)
end