Class: Module
- Includes:
- Roby::Models::TaskServiceDefinitionDSL
- Defined in:
- lib/roby/support.rb,
lib/roby/actions/library.rb,
lib/roby/distributed_object.rb,
lib/roby/transaction/proxying.rb,
lib/roby/droby/enable.rb
Direct Known Subclasses
Roby::Actions::Models::Library, Roby::Models::TaskServiceModel, Roby::Relations::Space
Instance Method Summary collapse
- #action_library(name = nil, &block) ⇒ Object
- #droby_dump(dest) ⇒ Object
- #each_fullfilled_model {|_self| ... } ⇒ Object
-
#private_model? ⇒ Boolean
There are currently no way to make a module private.
-
#proxy_for(real_klass) ⇒ Object
Declare that
proxy_klassshould be used to wrap objects ofreal_klass.
Methods included from Roby::Models::TaskServiceDefinitionDSL
Instance Method Details
#action_library(name = nil, &block) ⇒ Object
9 10 11 12 13 14 15 16 17 18 19 |
# File 'lib/roby/actions/library.rb', line 9 def action_library(name = nil, &block) if name create_and_register_submodel(self, name, Roby::Actions::Library, &block) else extend Roby::Actions::Models::Interface extend MetaRuby::ModelAsModule extend MetaRuby::Registration self.supermodel = Roby::Actions::Library Roby::Actions::Library.register_submodel(self) end end |
#droby_dump(dest) ⇒ Object
12 13 14 |
# File 'lib/roby/droby/enable.rb', line 12 def droby_dump(dest) raise "can't dump modules (#{self})" end |
#each_fullfilled_model {|_self| ... } ⇒ Object
32 33 34 35 |
# File 'lib/roby/support.rb', line 32 def each_fullfilled_model return enum_for(__method__) if !block_given? yield self end |
#private_model? ⇒ Boolean
There are currently no way to make a module private
25 |
# File 'lib/roby/distributed_object.rb', line 25 def private_model?; false end |
#proxy_for(real_klass) ⇒ Object
Declare that proxy_klass should be used to wrap objects of real_klass. Order matters: if more than one wrapping matches, we will use the one defined last.
5 6 7 |
# File 'lib/roby/transaction/proxying.rb', line 5 def proxy_for(real_klass) Roby::Transaction::.(self, real_klass) end |