Method: Roby::Distributed.on_transaction
- Defined in:
- lib/roby/distributed/transaction.rb
.on_transaction(&block) ⇒ Object
Sets up the transaction handler. The given block will be called in a separate thread whenever a remote peer proposes a new transaction
14 15 16 |
# File 'lib/roby/distributed/transaction.rb', line 14 def on_transaction(&block) Distributed.transaction_handler = block end |