Class: Roby::Distributed::CallSpec
- Defined in:
- lib/roby/distributed/communication.rb,
lib/roby/distributed/communication.rb
Overview
The specification of a call in Peer#send_queue and Peer#completion_queue. Note that only the #is_callback, #method and #formatted_args are sent to the remote PeerServer#demux method
-
is_callback is a boolean flag indicating if this call has been queued while the PeerServer object was processing a remote request
-
methodis the method name to call on the remote PeerServer object -
formatted_argsis the arguments formatted by Distributed.format. Arguments are formatted right away, since we want the marshalled arguments to reflect objects state at the time of the call, not at the time they are sent -
original_argsis the arguments not yet formatted. They are kept here to protect involved object from Ruby’s GC until the call is completed. -
on_completionis a proc object which will be called when the method has successfully been processed by the remote object, with the returned value as argument$ -
trace is the location (as returned by Kernel#caller) from which the call has been queued. It is mainly used for debugging purposes
-
if
threadis not nil, it is the thread which is waiting for the call to complete. If the call is aborted, the error will be raised in the waiting thread
Instance Attribute Summary collapse
-
#formatted_args ⇒ Object
Returns the value of attribute formatted_args.
-
#is_callback ⇒ Object
(also: #callback?)
Returns the value of attribute is_callback.
-
#message_id ⇒ Object
Returns the value of attribute message_id.
-
#method ⇒ Object
Returns the value of attribute method.
-
#on_completion ⇒ Object
Returns the value of attribute on_completion.
-
#original_args ⇒ Object
Returns the value of attribute original_args.
-
#trace ⇒ Object
Returns the value of attribute trace.
-
#waiting_thread ⇒ Object
Returns the value of attribute waiting_thread.
Instance Method Summary collapse
Instance Attribute Details
#formatted_args ⇒ Object
Returns the value of attribute formatted_args
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def formatted_args @formatted_args end |
#is_callback ⇒ Object Also known as: callback?
Returns the value of attribute is_callback
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def is_callback @is_callback end |
#message_id ⇒ Object
Returns the value of attribute message_id
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def @message_id end |
#method ⇒ Object
Returns the value of attribute method
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def method @method end |
#on_completion ⇒ Object
Returns the value of attribute on_completion
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def on_completion @on_completion end |
#original_args ⇒ Object
Returns the value of attribute original_args
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def original_args @original_args end |
#trace ⇒ Object
Returns the value of attribute trace
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def trace @trace end |
#waiting_thread ⇒ Object
Returns the value of attribute waiting_thread
421 422 423 |
# File 'lib/roby/distributed/communication.rb', line 421 def waiting_thread @waiting_thread end |