Libraries
»
roby
(0.7.2)
»
Index (N)
»
Roby
»
Distributed
»
NotEditor
Exception: Roby::Distributed::NotEditor
Inherits:
RuntimeError
Object
RuntimeError
Roby::Distributed::NotEditor
show all
Defined in:
lib/roby/distributed/transaction.rb
Overview
Raised when an operation needs the edition token, while the local plan manager does not have it.