Class: Roby::FilterGenerator
- Inherits:
-
EventGenerator
- Object
- BasicObject
- PlanObject
- EventGenerator
- Roby::FilterGenerator
- Defined in:
- lib/roby/event.rb
Overview
This generator reemits an event after having changed its context. See EventGenerator#filter for a more complete explanation
Constant Summary
Constants included from Log::EventGeneratorHooks
Log::EventGeneratorHooks::HOOKS
Constants included from Log::BasicObjectHooks
Instance Attribute Summary
Attributes inherited from EventGenerator
#command, #executable, #pending, #unreachable_handlers
Attributes inherited from PlanObject
#executable, #plan, #removed_at
Attributes inherited from BasicObject
Instance Method Summary collapse
-
#initialize(user_context, &block) ⇒ FilterGenerator
constructor
A new instance of FilterGenerator.
Methods inherited from EventGenerator
#&, #_dump, _load, #achieve_with, #add_child_object, #call, #call_handlers, #call_without_propagation, #called, #calling, #cancel, #controlable?, #default_command, #delay, #droby_dump, #each_precondition, #emit, #emit_failed, #emit_on, #emit_without_propagation, #emitting, event_gathering, #executable?, #filter, #fired, #forward, #forward_once, #forwarding, gather_events, #if_unreachable, #initialize_copy, #last, #model, #name, #new, #on, #once, #pending?, #postpone, #postponed, #precondition, #pretty_print, #realize_with, #related_events, #related_tasks, remove_event_gathering, #signal, #signal_once, #signalling, #to_event, #unreachable!, #until, #when_unreachable, #|
Methods included from Distributed::DRobyModel::Dump
Methods included from Distributed::EventNotifications
#fired, #forwarding, #signalling
Methods included from Propagation::EventPrecedenceChanged
#added_child_object, #removed_child_object
Methods included from Log::EventGeneratorHooks
#added_child_object, #called, #calling, #emitting, #fired, #forwarding, #postponed, #removed_child_object, #signalling
Methods inherited from PlanObject
#add_child_object, #apply_relation_changes, child_plan_object, #each_plan_child, #executable?, #finalized?, #forget_peer, #read_write?, #remotely_useful?, #removing_child_object, #replace_by, #replace_subplan_by, #root_object, #root_object?, #subscribed?, #update_on?, #updated_by?
Methods included from Distributed::RelationModificationHooks
#added_child_object, #removed_child_object
Methods included from Transactions::PlanObjectUpdates
#adding_child_object, #removing_child_object
Methods included from DirectedRelationSupport
#add_child_object, #add_parent_object, #check_is_relation, #related_objects, #relations, #remove_child_object, #remove_children, #remove_parent_object, #remove_parents, #remove_relations
Methods inherited from BasicObject
#add_sibling_for, #distribute?, distribute?, #finalized?, #forget_peer, #has_sibling_on?, #initialize_copy, local_only, #read_write?, #remotely_useful?, #remove_sibling_for, #self_owned?, #sibling_of, #sibling_on, #subscribe, #subscribed?, #update_on?, #updated?, #updated_by?, #updated_peers
Methods included from Log::BasicObjectHooks
Constructor Details
#initialize(user_context, &block) ⇒ FilterGenerator
Returns a new instance of FilterGenerator.
756 757 758 759 760 761 762 763 764 765 766 767 768 769 770 771 772 773 |
# File 'lib/roby/event.rb', line 756 def initialize(user_context, &block) if block && !user_context.empty? raise ArgumentError, "you must set either the filter or the value, not both" end if block super() do |context| context = context.map do |val| block.call(val) end emit(*context) end else super() do emit(*user_context) end end end |