Class: Roby::Log::ChronicleDisplay::Line
- Defined in:
- lib/roby/log/chronicle.rb,
lib/roby/log/chronicle.rb
Overview
- y
-
the Y of this line in the graphics scene
- item
-
the main item, either a free event or a task
- graphic_items
-
the array of graphic items
- graphics_group
-
a Qt::GraphicsGroup object holding all graphic objects for this line
Instance Attribute Summary collapse
-
#graphic_group ⇒ Object
Returns the value of attribute graphic_group.
-
#graphic_items ⇒ Object
Returns the value of attribute graphic_items.
-
#item ⇒ Object
Returns the value of attribute item.
-
#start_time ⇒ Object
Returns the value of attribute start_time.
-
#y ⇒ Object
Returns the value of attribute y.
Instance Method Summary collapse
Instance Attribute Details
#graphic_group ⇒ Object
Returns the value of attribute graphic_group
13 14 15 |
# File 'lib/roby/log/chronicle.rb', line 13 def graphic_group @graphic_group end |
#graphic_items ⇒ Object
Returns the value of attribute graphic_items
13 14 15 |
# File 'lib/roby/log/chronicle.rb', line 13 def graphic_items @graphic_items end |
#item ⇒ Object
Returns the value of attribute item
13 14 15 |
# File 'lib/roby/log/chronicle.rb', line 13 def item @item end |
#start_time ⇒ Object
Returns the value of attribute start_time
13 14 15 |
# File 'lib/roby/log/chronicle.rb', line 13 def start_time @start_time end |
#y ⇒ Object
Returns the value of attribute y
13 14 15 |
# File 'lib/roby/log/chronicle.rb', line 13 def y @y end |
Instance Method Details
#add(graphic_item, time) ⇒ Object
15 16 17 18 19 20 21 22 23 24 |
# File 'lib/roby/log/chronicle.rb', line 15 def add(graphic_item, time) class << graphic_item; attr_accessor :time end graphic_item.time = time graphic_group.add_to_group graphic_item if graphic_items.empty? self.start_time = time end graphic_items << graphic_item end |