Method: Roby::Log::RelationsDisplay#execution_events
- Defined in:
- lib/roby/log/relations.rb
#execution_events ⇒ Object (readonly)
The array of events for which a command has been called, or which have been emitted. The order in this array is the arrival order of the corresponding events.
An array element is [fired, event], when fired is true if the event has been fired, and false if it is pending
446 447 448 |
# File 'lib/roby/log/relations.rb', line 446 def execution_events @execution_events end |