Class: Roby::Task::InternalError
Deprecated Event API collapse
-
#failure? ⇒ Boolean
Mark the InternalError event as a failure event, even if it is not forwarded to the stop event at the model level.
Instance Method Details
#failure? ⇒ Boolean
Mark the InternalError event as a failure event, even if it is not forwarded to the stop event at the model level
976 977 978 |
# File 'lib/roby/task.rb', line 976 def failure? true end |