Class: Roby::TimePointEvent
- Inherits:
-
StateEvent
- Object
- BasicObject
- PlanObject
- EventGenerator
- StateEvent
- Roby::TimePointEvent
- Defined in:
- lib/roby/state/events.rb
Overview
This event emits itself when the specified time is reached
Constant Summary
Constants included from Log::EventGeneratorHooks
Log::EventGeneratorHooks::HOOKS
Constants included from Log::BasicObjectHooks
Instance Attribute Summary collapse
-
#time ⇒ Object
readonly
Time at which this event should emit himself.
Attributes inherited from EventGenerator
#command, #executable, #pending, #unreachable_handlers
Attributes inherited from PlanObject
#executable, #plan, #removed_at
Attributes inherited from BasicObject
Instance Method Summary collapse
-
#initialize(time) ⇒ TimePointEvent
constructor
Creates an event which will emit when
timeis reached. -
#poll ⇒ Object
Called at each cycle by Roby.poll_state_events.
Methods inherited from StateEvent
#disable, #disabled?, #enable, #enabled?
Methods inherited from EventGenerator
#&, #_dump, _load, #achieve_with, #add_child_object, #call, #call_handlers, #call_without_propagation, #called, #calling, #cancel, #controlable?, #default_command, #delay, #droby_dump, #each_precondition, #emit, #emit_failed, #emit_on, #emit_without_propagation, #emitting, event_gathering, #executable?, #filter, #fired, #forward, #forward_once, #forwarding, gather_events, #if_unreachable, #initialize_copy, #last, #model, #name, #new, #on, #once, #pending?, #postpone, #postponed, #precondition, #pretty_print, #realize_with, #related_events, #related_tasks, remove_event_gathering, #signal, #signal_once, #signalling, #to_event, #unreachable!, #until, #when_unreachable, #|
Methods included from Distributed::DRobyModel::Dump
Methods included from Distributed::EventNotifications
#fired, #forwarding, #signalling
Methods included from Propagation::EventPrecedenceChanged
#added_child_object, #removed_child_object
Methods included from Log::EventGeneratorHooks
#added_child_object, #called, #calling, #emitting, #fired, #forwarding, #postponed, #removed_child_object, #signalling
Methods inherited from PlanObject
#add_child_object, #apply_relation_changes, child_plan_object, #each_plan_child, #executable?, #finalized?, #forget_peer, #read_write?, #remotely_useful?, #removing_child_object, #replace_by, #replace_subplan_by, #root_object, #root_object?, #subscribed?, #update_on?, #updated_by?
Methods included from Distributed::RelationModificationHooks
#added_child_object, #removed_child_object
Methods included from Roby::Transactions::PlanObjectUpdates
#adding_child_object, #removing_child_object
Methods included from DirectedRelationSupport
#add_child_object, #add_parent_object, #check_is_relation, #related_objects, #relations, #remove_child_object, #remove_children, #remove_parent_object, #remove_parents, #remove_relations
Methods inherited from BasicObject
#add_sibling_for, #distribute?, distribute?, #finalized?, #forget_peer, #has_sibling_on?, #initialize_copy, local_only, #read_write?, #remotely_useful?, #remove_sibling_for, #self_owned?, #sibling_of, #sibling_on, #subscribe, #subscribed?, #update_on?, #updated?, #updated_by?, #updated_peers
Methods included from Log::BasicObjectHooks
Constructor Details
#initialize(time) ⇒ TimePointEvent
Creates an event which will emit when time is reached
98 99 100 101 |
# File 'lib/roby/state/events.rb', line 98 def initialize(time) @time = time super end |
Instance Attribute Details
#time ⇒ Object (readonly)
Time at which this event should emit himself
95 96 97 |
# File 'lib/roby/state/events.rb', line 95 def time @time end |
Instance Method Details
#poll ⇒ Object
Called at each cycle by Roby.poll_state_events
104 105 106 107 108 |
# File 'lib/roby/state/events.rb', line 104 def poll # :nodoc: if !happened? && Time.now >= time emit end end |